'''
Created on 31.01.2011

@author: steffen.siebert
'''
import getopt
import struct
import sys
from zlib import crc32

UBL_MAGIC_BIN_IMG = 0xA1ACED66
""" Describes the application image in Flash - indicates that it is binary """
UBL_LOAD_ADDR       = 0x83fc0000
UBL_ENTRY_POINT     = 0x83fc0000
UBL_MAGIC_BYTES     = struct.pack("I", UBL_MAGIC_BIN_IMG)
UBL_HEADER_LENGTH = 24
UBL_HEADER_FORMAT = "<IIIIII"

ROM_LOAD_HDR_STRING = "LoadHdrStart"
ROM_LOAD_ADDR       = 0x2000302c
ROM_ENTRY_POINT     = 0x2000302c
ROM_HEADER_LENGTH = 36
ROM_HEADER_FORMAT = "<16sIIIII" 
ROM_HEADER_SEGMENT_LENGTH = 8
ROM_HEADER_SEGMENT_FORMAT = "<II" 

def createUBLHeader(content, entryPoint, loadAddr):
    size = len(content)
    # crc32 returns signed integers, so make it unsigned.
    crc = crc32(content) & 0xffffffffL
    print "crc: 0x%08x" % crc
    return struct.pack(UBL_HEADER_FORMAT, UBL_MAGIC_BIN_IMG, entryPoint, loadAddr, size, 24, crc)

def makeUBLImage(fileName, entryPoint, loadAddr):
    outFileName = fileName + ".ublimg"
    f = open(fileName, "rb")
    content = f.read()
    f.close()

    print "Input %s: size: %d bytes\n" % (fileName, len(content))

    header = createUBLHeader(content, entryPoint, loadAddr)

    print "Output %s: size: %d bytes\n" % (outFileName, len(header) + len(content))

    f = open(outFileName, "wb")
    f.write(header)
    f.write(content)
    f.close()

def createROMHeader(content, entryPoint, loadAddr):
    size = len(content)
    type = 1        # Load type: Secondary boot
    override = 0    # Security override
    headerSize = 44 # Load header size (given there is only 1 segment)
    segments = 1    # Segment count
    header = struct.pack(ROM_HEADER_FORMAT, ROM_LOAD_HDR_STRING, type, entryPoint, override, headerSize, segments)
    segmentHeader = struct.pack(ROM_HEADER_SEGMENT_FORMAT, size, loadAddr)
    return header + segmentHeader

def makeROMImage(fileName, entryPoint, loadAddr):
    outFileName = fileName + ".romimg"
    f = open(fileName, "rb")
    content = f.read()
    f.close()

    print "Input %s: size: %d bytes\n" % (fileName, len(content))

    header = createROMHeader(content, entryPoint, loadAddr)

    print "Output %s: size: %d bytes\n" % (outFileName, len(header) + len(content))

    f = open(outFileName, "wb")
    f.write(header)
    f.write(content)
    f.close()

def showHeader(fileName):
    f = open(fileName, "rb")
    content = f.read()
    f.close()

    print "Input %s: size: %d bytes\n" % (fileName, len(content))
    
    if content[:len(ROM_LOAD_HDR_STRING)] == ROM_LOAD_HDR_STRING:
        showROMHeader(content)
    else:
        showUBLHeader(content)

def showROMHeader(content):
    values = struct.unpack(ROM_HEADER_FORMAT, content[:ROM_HEADER_LENGTH])
    headerString = values[0].strip(chr(0x00))

    if headerString == ROM_LOAD_HDR_STRING:
        print "[Image Header]"
        print "  Hdr String        : %s" % headerString
        print "  Load Type         : 0x%08X" % values[1]
        print "  Start Address     : 0x%08X" % values[2]
        print "  Security Override : %d" % values[3]
        print "  Load Header Size  : %d" % values[4]
        print "  Segment Count     : %d" % values[5]
        content = content[ROM_HEADER_LENGTH:]
        for i in range(values[5]):
            segValues = struct.unpack(ROM_HEADER_SEGMENT_FORMAT, content[i*ROM_HEADER_SEGMENT_LENGTH:(i+1)*ROM_HEADER_SEGMENT_LENGTH])
            print "   %d# addr:0x%08X  len:%d" % (i, segValues[1], segValues[0])
    else:
        print "ROM Load Header not found."
        print "Expecting %s found %s" % (ROM_LOAD_HDR_STRING, headerString)

def showUBLHeader(content):
    values = struct.unpack(UBL_HEADER_FORMAT, content[:UBL_HEADER_LENGTH])
    # crc32 returns signed integers, so make it unsigned.
    crc = crc32(content[UBL_HEADER_LENGTH:]) & 0xffffffffL

    if values[0] == UBL_MAGIC_BIN_IMG:
        print "[Image Header]"
        print "  Magic Num   : 0x%08X" %  values[0]
        print "  Entry Point : 0x%08X" %  values[1]
        print "  Load Address: 0x%08X" %  values[2]
        print "  Image Size  : %d" %  values[3]
        print "  Hdr Size    : %d" %  values[4]
        print "  CRC32       : 0x%08X" %  values[5]
        print "[Calculated]"
        print "  Image Size  : %d" % (len(content) - UBL_HEADER_LENGTH)
        print "  CRC32       : 0x%08X" % crc
    else:
        print "UBL Header not found."
        print "Expecting %x found %x" % (UBL_MAGIC_BIN_IMG, values[0])

def usage():
    print "Usage: %s -lrh <image_file>" % sys.argv[0]
    print "-l : show load header from image."
    print "-r : Handle ROM image instead of UBL image."
    print "-h : Show help."

def main():
    printHeader = False
    ublImage = True

    try:
        opts, args = getopt.getopt(sys.argv[1:], "lrh")
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)
    for o, _ in opts:
        if o == "-l":
            printHeader = True
        elif o == "-r":
            ublImage = False
        elif o == "-h":
            usage()
            sys.exit()
        else:
            print "Unknown option %s" % o
            usage()
            sys.exit(2)

    if len(args) > 1:
        print "Too many arguments: %s" % args
        usage()
        sys.exit(2)
    elif len(args) == 0:
        print "No image file defined"
        usage()
        sys.exit(2)

    fileName = args[0]
    if printHeader:
        showHeader(fileName)
    else:
        if ublImage:
            makeUBLImage(fileName, UBL_ENTRY_POINT, UBL_LOAD_ADDR)
        else:
            makeROMImage(fileName, ROM_ENTRY_POINT, ROM_LOAD_ADDR)

if __name__ == '__main__':
    main()
